SystemOrganization addCategory: #'AST-Interpreter-Test'!

TestCase subclass: #ASTInterpreterTest
	instanceVariableNames: 'interpreter context'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AST-Interpreter-Test'!

!ASTInterpreterTest class methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/9/2011 19:56'!
testMethodArguments: anObject
	^ [ anObject ] value! !

!ASTInterpreterTest methodsFor: 'helper' stamp: 'CamilloBruni 12/5/2011 19:01'!
compile: aString
	| ast |
	ast := RBParser parseExpression: aString.
	ast annotateInClass: ProtoObject.
	^ ast! !

!ASTInterpreterTest methodsFor: 'helper' stamp: 'CamilloBruni 12/5/2011 20:17'!
interpret: aString
	| result |
	
	interpreter resetContext: context.
	result := interpreter interpret: (self compile: aString).
	
	self assert: interpreter context = context.
	^ result! !

!ASTInterpreterTest methodsFor: 'helper' stamp: 'CamilloBruni 12/9/2011 14:04'!
setUp
	"for now we use an ast cache to speed up the ast lookup"
	ASTCache initialize.
	interpreter := ASTInterpreter new.
	context := AIMethodContext new.! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/9/2011 19:53'!
testBasicCode
	self assert: (self interpret: '[ true ifTrue: [ nil ] ifFalse: [ 1 ]] value') = nil.
	self assert: (self interpret: '[ false ifTrue: [ nil ] ifFalse: [ 1 ]] value') = 1.
	
	self assert: (self interpret: '[ |a| a :=1. [ a < 10 ] whileTrue: [ a := a + 1]. a] value') = 10.! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 10/5/2011 17:39'!
testBinarySend
	self assert: (self interpret: '1 + 2') = 3.
	self assert: (self interpret: '1 < 2') = true.! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/5/2011 18:26'!
testBlock
	self assert: ((self interpret: '[ 1 ]') isKindOf: ASTBlockClosure).
	
	self assert: (self interpret: '[ 1 ] value') = 1.
	self assert: (self interpret: '[ 1 + 2 ] value') = 3.
	
	self assert: (self interpret: 'true ifTrue: [ 1 ] ifFalse: [ 0 ]') = 1.
	self assert: (self interpret: 'false ifTrue: [ 1 ] ifFalse: [ 0 ]') = 0.
	! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/5/2011 18:10'!
testBlockArgument

	self assert: (self interpret: '[ :a| a ] value: 1') = 1.
	self assert: (self interpret: '[ :a| a + 3 ] value: 3 ') = 6.! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/8/2011 15:31'!
testBlockVar

	self assert: (self interpret: '[ |a| a := 1. a ] value') = 1.
	self assert: (self interpret: '[ |a| a := 1 + 2. a + 3 ] value') = 6.
	
	self assert: (self interpret: '[ |a| [ a := 1 ] value ] value') = 1.
	self assert: (self interpret: '[ |a| a := 1. [ a := 2 ] value ] value') = 2.
	
	self assert: (self interpret: '[ |a| a := 1. [ a := a + 1 ] value ] value') = 2.
	
	self assert: (self interpret: '[ :b ||a| a := 1. [ :c| a := a + 1 + c ] value: b ] value: 3') = 5.! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/5/2011 09:56'!
testConstant
	self assert: (self interpret: '1') = 1.
	self assert: (self interpret: 'nil') = nil.
	self assert: (self interpret: 'false') = false.
	self assert: (self interpret: '''asdf''') = 'asdf'.
	self assert: (self interpret: '#symbol') = #symbol.! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/9/2011 14:06'!
testDNU
	self fail.
	self 
		should: (self interpret: '1 aMessageNotUnderstoodBySmallInteger') 
		raise: Exception.! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/9/2011 15:31'!
testException
	self fail.
	self 
		should: (self interpret: 'Error signal') 
		raise: InterpretationError.! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/7/2011 18:29'!
testExceptionHandling

	self fail.
	self assert: (self interpret: '[ ] on: Error do: [ :err| false ]. true') = true.
	self assert: (self interpret: '[ Error signal ] on: Error do: [ :err| false ]. true') = false.! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/5/2011 14:56'!
testGlobals
	self assert: (self interpret: 'Array') = Array! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/8/2011 18:19'!
testInstanceCreation
	self assert: ((self interpret: 'Array new') isKindOf: Array).
	self assert: ((self interpret: 'Error new') isKindOf: Error).
	self assert: ((self interpret: 'MessageNotUnderstood new') isKindOf: MessageNotUnderstood)! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/9/2011 19:55'!
testMethodArguments
	
	self assert: (self interpret: 'ASTInterpreterTest testMethodArguments: 1') = 1.
	! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/8/2011 18:21'!
testNonLocalReturn

	self assert: (self interpret: 'false ifTrue: [ ^ 1 ]. ^ 1 + 1') = 2.
	self assert: (self interpret: 'true ifTrue: [ ^ 1 ]. ^ 1 + 1') = 1.
	
	self assert: (self interpret: 'false ifTrue: [ ^ 1 ] ifFalse: [^ 2]. 1 + 1 + 1') = 2.! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/8/2011 00:21'!
testPrimitive
	self assert: (self interpret: '1 + 1') = 2. "simple"
	self assert: (self interpret: '1 + 1.5') = 2.5. "with fallback"! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/5/2011 19:43'!
testSend
	self assert: (self interpret: '#(1 2) at: 1') = 1.
	self assert: (self interpret: '1 + 2') = 3.
	"this is for future use :)"
	"self assert: (self interpret: '123 asString') = '123'."! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/5/2011 11:24'!
testUnarySend
	self assert: (self interpret: '1 asInteger') = 1.
	self assert: (self interpret: '1 class') = SmallInteger.! !
